/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s):
 *   Peter Hartshorn <peter@igelaus.com.au>
 *   B.J. Rossiter <bj@igelaus.com.au>
 */

#ifndef nsDrawingSurfaceNanoX_h__
#define nsDrawingSurfaceNanoX_h__

#include "nsIDrawingSurface.h"

#include <nano-X.h>

class nsDrawingSurfaceNanoX : public nsIDrawingSurface
{
public:
  nsDrawingSurfaceNanoX();
  virtual ~nsDrawingSurfaceNanoX();

  NS_DECL_ISUPPORTS

  NS_IMETHOD Lock(PRInt32 aX, PRInt32 aY, PRUint32 aWidth, PRUint32 aHeight,
                  void **aBits, PRInt32 *aStride, PRInt32 *aWidthBytes,
                  PRUint32 aFlags);
  NS_IMETHOD Unlock(void);
  NS_IMETHOD GetDimensions(PRUint32 *aWidth, PRUint32 *aHeight);
  NS_IMETHOD IsOffscreen(PRBool *aOffScreen);
  NS_IMETHOD IsPixelAddressable(PRBool *aAddressable);
  NS_IMETHOD GetPixelFormat(nsPixelFormat *aFormat);

  NS_IMETHOD Init(GR_WINDOW_ID aDrawable);
  NS_IMETHOD Init(PRUint32 aWidth, PRUint32 aHeight, PRUint32 aFlags);
  NS_IMETHOD SetRegion(GR_REGION_ID aRegion);
  GR_GC_ID GetGC(void) { return mGC; }
  GR_WINDOW_ID GetDrawable(void) { return mDrawable; }
  void GetSize(PRUint32 *aWidth, PRUint32 *aHeight) { *aWidth = mWidth; *aHeight = mHeight; }
  int GetDepth() { return mDepth; }

private:
  static PRUint8 ConvertMaskToCount(unsigned long val);
  static PRUint8 GetShiftForMask(unsigned long val);

  int mDepth;
  GR_GC_ID mGC;
  GR_WINDOW_ID mDrawable;
  GR_REGION_ID mRegion;
  nsPixelFormat mPixFormat;

  PRInt32 mLockX;
  PRInt32 mLockY;
  PRUint32 mLockWidth;
  PRUint32 mLockHeight;
  PRUint32 mLockFlags;
  PRBool mLocked;
  GR_PIXELVAL *pixels;

  PRUint32 mWidth;
  PRUint32 mHeight;

  PRBool mIsOffscreen;
  PRBool mOurDrawable;
};

#endif
