/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s):
 *   Peter Hartshorn <peter@igelaus.com.au>
 *   B.J. Rossiter <bj@igelaus.com.au>
 */

#include "prmem.h"
#include "nsRegionNanoX.h"

static NS_DEFINE_IID(kRegionIID, NS_IREGION_IID);
static int nregions = 0;

nsRegionNanoX::nsRegionNanoX()
{
  NS_INIT_REFCNT();

  mRegion = 0;
  mRegionType = eRegionComplexity_empty;
}

nsRegionNanoX::~nsRegionNanoX()
{
  if (mRegion)
    GdDestroyRegion(mRegion);

  mRegion = 0;
}

NS_IMPL_QUERY_INTERFACE(nsRegionNanoX, kRegionIID)
NS_IMPL_ADDREF(nsRegionNanoX)
NS_IMPL_RELEASE(nsRegionNanoX)

nsresult
nsRegionNanoX::Init()
{
  if (mRegion)
    GdDestroyRegion(mRegion);

  mRegion = ::GdAllocRegion();
  mRegionType = eRegionComplexity_empty;

  return NS_OK;
}

void
nsRegionNanoX::SetTo(const nsIRegion &aRegion)
{
  nsRegionNanoX * pRegion = (nsRegionNanoX *)&aRegion;
 
  SetRegionEmpty(); 
 
  MWCLIPREGION *nRegion = 	::GdAllocRegion();
  /* nano switches dest/src order */
  GdUnionRegion(nRegion, pRegion->mRegion, mRegion);
  GdDestroyRegion(mRegion);
  mRegion = nRegion;
}

void
nsRegionNanoX::SetTo(PRInt32 aX, PRInt32 aY, PRInt32 aWidth, PRInt32 aHeight)
{
  SetRegionEmpty();

  GdDestroyRegion(mRegion);
  mRegion = GdAllocRectRegion(aX, aY, aX + aWidth, aY + aHeight);
}

void
nsRegionNanoX::Intersect(const nsIRegion &aRegion)
{
  nsRegionNanoX * pRegion = (nsRegionNanoX *)&aRegion;
  
  MWCLIPREGION *nRegion = ::GdAllocRegion();
  GdIntersectRegion(nRegion, pRegion->mRegion, mRegion);
  GdDestroyRegion(mRegion);
  mRegion = nRegion;
}

void
nsRegionNanoX::Intersect(PRInt32 aX, PRInt32 aY, PRInt32 aWidth, PRInt32 aHeight)
{
  MWCLIPREGION *tRegion = GdAllocRectRegion(aX, aY, aX + aWidth, aY + aHeight);
  
  MWCLIPREGION *nRegion = ::GdAllocRegion();

  GdIntersectRegion(nRegion, tRegion, mRegion);
  GdDestroyRegion(tRegion);
  GdDestroyRegion(mRegion);
  mRegion = nRegion;
}

void
nsRegionNanoX::Union(const nsIRegion &aRegion)
{
   nsRegionNanoX * pRegion = (nsRegionNanoX *)&aRegion;
 
   MWCLIPREGION *nRegion = ::GdAllocRegion();
   GdUnionRegion(nRegion, pRegion->mRegion, mRegion);
   GdDestroyRegion(mRegion);
   mRegion = nRegion;
}

void
nsRegionNanoX::Union(PRInt32 aX, PRInt32 aY, PRInt32 aWidth, PRInt32 aHeight)
{
  MWCLIPREGION *tRegion = CreateRectRegion(aX, aY, aWidth, aHeight);
 
  MWCLIPREGION *nRegion = ::GdAllocRegion();
  GdUnionRegion(nRegion, tRegion, mRegion);
  GdDestroyRegion(mRegion);
  GdDestroyRegion(tRegion);
  mRegion = nRegion;
}

void
nsRegionNanoX::Subtract(const nsIRegion &aRegion)
{
  nsRegionNanoX * pRegion = (nsRegionNanoX *)&aRegion;
  
  MWCLIPREGION *nRegion = ::GdAllocRegion();
  GdIntersectRegion(nRegion, mRegion, pRegion->mRegion);
  GdSubtractRegion(nRegion, mRegion, nRegion);
  GdDestroyRegion(mRegion);
  mRegion = nRegion;
}

void
nsRegionNanoX::Subtract(PRInt32 aX, PRInt32 aY, PRInt32 aWidth, PRInt32 aHeight)
{
  MWCLIPREGION *tRegion = CreateRectRegion(aX, aY, aWidth, aHeight);
  
  MWCLIPREGION *nRegion = ::GdAllocRegion();
  GdSubtractRegion(nRegion,mRegion, tRegion);
  GdDestroyRegion(mRegion);
  GdDestroyRegion(tRegion);
  mRegion = nRegion;
}

PRBool
nsRegionNanoX::IsEmpty(void)
{
  return GdEmptyRegion(mRegion);
}

PRBool
nsRegionNanoX::IsEqual(const nsIRegion &aRegion)
{
  nsRegionNanoX *pRegion = (nsRegionNanoX *)&aRegion;
  
  return GdEqualRegion(mRegion, pRegion->mRegion);
}

void
nsRegionNanoX::GetBoundingBox(PRInt32 *aX, PRInt32 *aY,
                             PRInt32 *aWidth, PRInt32 *aHeight)
{
  MWRECT r;
  /* make sure nano-x code is not broken (still) ie returns empty region*/
  GdGetRegionBox(mRegion, &r);

  *aX = r.left;
  *aY = r.top;
  *aWidth = r.right - r.left;
  *aHeight = r.bottom - r.top;
}

void
nsRegionNanoX::Offset(PRInt32 aXOffset, PRInt32 aYOffset)
{
  GdOffsetRegion(mRegion, aXOffset, aYOffset);
}

PRBool
nsRegionNanoX::ContainsRect(PRInt32 aX, PRInt32 aY,
                           PRInt32 aWidth, PRInt32 aHeight)
{
  /* XXX  width or coordinates for last two args?? */
  MWRECT r;
  r.left = aX;
  r.top = aY;
  r.right = aX + aWidth;
  r.bottom = aY + aHeight;
  return GdRectInRegion(mRegion, &r);
}

NS_IMETHODIMP
nsRegionNanoX::GetRects(nsRegionRectSet **aRects)
{
  nsRegionRectSet   *rects;
  int               nbox;
  MWRECT            *pbox;  
  nsRegionRect      *rect;
  NS_ASSERTION(!(nsnull == aRects), "bad ptr");
  //code lifted from old xfe. MMP
  
  pbox = mRegion->rects;
  nbox = mRegion->numRects;
 
  rects = *aRects;

  if ((nsnull == rects) || (rects->mRectsLen < (PRUint32)nbox))
  {
    void *buf = PR_Realloc(rects, sizeof(nsRegionRectSet) + (sizeof(nsRegionRect
) * (nbox - 1)));
    
    if (nsnull == buf)
    {
      if (nsnull != rects)
        rects->mNumRects = 0;
 
      return NS_OK;
    }
  
    rects = (nsRegionRectSet *)buf;
    rects->mRectsLen = nbox;
  }

  rects->mNumRects = nbox;
  rects->mArea = 0;
  rect = &rects->mRects[0];

  while (nbox--)
  {
    rect->x = pbox->left;
    rect->width = (pbox->right - pbox->left);
    rect->y = pbox->top;
    rect->height = (pbox->bottom - pbox->top);

    rects->mArea += rect->width * rect->height;

    pbox++;
    rect++;
  }

  *aRects = rects;
  return NS_OK;
}

NS_IMETHODIMP
nsRegionNanoX::FreeRects(nsRegionRectSet *aRects)
{
  if (nsnull != aRects)
    PR_Free((void *)aRects);

  return NS_OK;
}

NS_IMETHODIMP
nsRegionNanoX::GetNativeRegion(void *&aRegion) const
{
  GR_REGION_ID region;
  GR_RECT rect;
  int num;
  MWRECT *box;

  num = mRegion->numRects;
  box = mRegion->rects;

  region = GrNewRegion();

  while (num--) {
    rect.x = box->left;
    rect.y = box->top;
    rect.width = box->right - box->left;
    rect.height = box->bottom - box->top;
    GrUnionRectWithRegion(region, &rect);
    box++;
  }

  aRegion = (void *)region;
  return NS_OK;
}

NS_IMETHODIMP
nsRegionNanoX::GetRegionComplexity(nsRegionComplexity &aComplexity) const
{
  // cast to avoid const-ness problems on some compilers
  if (((nsRegionNanoX*)this)->IsEmpty())
    aComplexity = eRegionComplexity_empty;
  else 
    aComplexity = eRegionComplexity_rect;
      
  return NS_OK;
}

void nsRegionNanoX::SetRegionEmpty()
{ 
  if (!IsEmpty()) { 
    GdDestroyRegion(mRegion);
    mRegion = ::GdAllocRegion();
  }
}

MWCLIPREGION *
nsRegionNanoX::CreateRectRegion(PRInt32 aX,
                               PRInt32 aY,
                               PRInt32 aWidth,
                               PRInt32 aHeight)
{
  return GdAllocRectRegion(aX, aY, aX + aWidth, aY + aHeight);
} 
