/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s):
 *   Peter Hartshorn <peter@igelaus.com.au>
 *   B.J. Rossiter <bj@igelaus.com.au>
 *   Matthew Wlazlo <mattw@igelaus.com.au>
 */

#include <nsCOMPtr.h>
#include <nsAppShell.h>
#include <nsIAppShell.h>
#include <nsIServiceManager.h>
#include <nsIEventQueueService.h>
#include <nsICmdLineService.h>
#include <nsWidget.h>
#include <nsWidgetsCID.h>
#include <nsWindow.h>
#include <nsKeyCode.h>
#include <nsITimer.h>
#include <nsINanoXWindowService.h>
#include <nsIDragService.h>
#include <nsIDragSessionNanoX.h>

#include <nano-X.h>
#include <stdlib.h>
#include <stdio.h>

//
// XPCOM CIDs
//
//-------------------------------------------------------------------------
static NS_DEFINE_CID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);
static NS_DEFINE_IID(kEventQueueServiceIID, NS_IEVENTQUEUESERVICE_IID);
static NS_DEFINE_CID(kCmdLineServiceCID, NS_COMMANDLINE_SERVICE_CID);
static NS_DEFINE_CID(kCDragServiceCID, NS_DRAGSERVICE_CID);

#ifndef MOZ_MONOLITHIC_TOOLKIT
static NS_DEFINE_CID(kWindowServiceCID, NS_NANOX_WINDOW_SERVICE_CID);
static NS_DEFINE_IID(kWindowServiceIID, NS_NANOX_WINDOW_SERVICE_IID);
#endif

PRInt32 nsAppShell::mButton = 0;
PRBool nsAppShell::mDragging = PR_FALSE;
PRBool nsAppShell::mPleaseClose = PR_FALSE;
PRBool nsAppShell::mIsInitialized = PR_FALSE;
PRUint32 nsAppShell::mAppShellCount = 0;

PRBool nsAppShell::mClicked = PR_FALSE;
PRTime nsAppShell::mClickTime = 0;
PRInt8 nsAppShell::mClicks = 1;
PRUint16 nsAppShell::mClickedButton = 0;

PRBool nsAppShell::mShiftDown = PR_FALSE;
PRBool nsAppShell::mCtrlDown = PR_FALSE;
PRBool nsAppShell::mAltDown = PR_FALSE;

#ifndef MOZ_MONOLITHIC_TOOLKIT
extern "C" int NS_TimeToNextTimeout(struct timeval *);
extern "C" void NS_ProcessTimeouts(void);
#endif

static void nanoXErrorCallback(GR_EVENT *ep)
{
}

static GR_FNCALLBACKEVENT ErrorCallback = nanoXErrorCallback;

#ifndef MOZ_MONOLITHIC_TOOLKIT
static nsNanoXTimeToNextTimeoutFunc GetTimeToNextTimeoutFunc(void)
{
  static nsNanoXTimeToNextTimeoutFunc sFunc = nsnull;

  if (!sFunc) {
    nsINanoXWindowService *nanoXWindowService = nsnull;
    nsresult rv = nsServiceManager::GetService(kWindowServiceCID,
                                           kWindowServiceIID,
                                           (nsISupports**)&nanoXWindowService);
    NS_ASSERTION(NS_SUCCEEDED(rv), "Couldn't obtain window service.");

    if (NS_OK == rv && nsnull != nanoXWindowService) {
      nanoXWindowService->GetTimeToNextTimeoutFunc(&sFunc);
      NS_RELEASE(nanoXWindowService);
    }
  }
  return sFunc;
}

static nsNanoXProcessTimeoutsProc GetProcessTimeoutProc(void)
{
  static nsNanoXProcessTimeoutsProc sProc = nsnull;

  if (!sProc) {
    nsINanoXWindowService *nanoXWindowService = nsnull;
    nsresult rv = nsServiceManager::GetService(kWindowServiceCID,
                                           kWindowServiceIID,
                                           (nsISupports**)&nanoXWindowService);
    NS_ASSERTION(NS_SUCCEEDED(rv), "Couldn't obtain window service.");

    if (NS_OK == rv && nsnull != nanoXWindowService) {
      nanoXWindowService->GetProcessTimeoutsProc(&sProc);
      NS_RELEASE(nanoXWindowService);
    }
  }
  return sProc;
}
#endif

static int CallTimeToNextTimeoutFunc(struct timeval *aTimeval)
{
#ifdef MOZ_MONOLITHIC_TOOLKIT
  NS_TimeToNextTimeout(aTimeval);
#else
  nsNanoXTimeToNextTimeoutFunc func = GetTimeToNextTimeoutFunc();

  if (func) {
    return (*func)(aTimeval);
  }
  return 0;
#endif
}

static void CallProcessTimeoutsProc(void)
{
#ifdef MOZ_MONOLITHIC_TOOLKIT
  NS_ProcessTimeouts();
#else
  nsNanoXProcessTimeoutsProc proc = GetProcessTimeoutProc();

  if (proc) {
    (*proc)();
  }
#endif
}

nsAppShell::nsAppShell()
{
//  NS_INIT_REFCNT();
  NS_INIT_ISUPPORTS();
#ifdef DEBUG_peter
  fprintf(stderr, "nsAppShell::nsAppShell\n");
#endif
  mEventQueue = nsnull;
  mModalEventQueue = nsnull;
}

nsAppShell::~nsAppShell()
{
  GrFlush();
  //GrClose();
}


NS_IMPL_THREADSAFE_ISUPPORTS1(nsAppShell, nsIAppShell)

NS_IMETHODIMP nsAppShell::SetDispatchListener(nsDispatchListener* aDispatchListener)
{
  return NS_OK;
}

NS_IMETHODIMP nsAppShell::Create(int *bac, char **bav)
{
#ifdef DEBUG_peter
  fprintf(stderr, "nsAppShell::Create %d\n",mIsInitialized);
#endif

  if (mIsInitialized == PR_TRUE)
    return NS_OK;

  mIsInitialized = PR_TRUE;

  GrOpen();
  GrSetErrorHandler(ErrorCallback);

  return NS_OK;
}

NS_IMETHODIMP nsAppShell::Spinup()
{
  nsresult rv = NS_OK;

  nsCOMPtr<nsIEventQueueService> eventQService = do_GetService(kEventQueueServiceCID, &rv);

  if (NS_FAILED(rv)) {
    return rv;
  }

  rv = eventQService->GetThreadEventQueue(NS_CURRENT_THREAD, getter_AddRefs(mEventQueue));

  if (mEventQueue)
    return NS_OK;

  rv = eventQService->CreateThreadEventQueue();
  if (NS_FAILED(rv)) {
    return rv;
  }

  rv = eventQService->GetThreadEventQueue(NS_CURRENT_THREAD, getter_AddRefs(mEventQueue));

  return rv;
}

NS_IMETHODIMP nsAppShell::Run()
{
  GR_EVENT event;
  int queue_fd = -1;
  struct timeval cur_time;

  if (mEventQueue == nsnull)
    Spinup();
  if (mEventQueue == nsnull)
    return NS_ERROR_NOT_INITIALIZED;

  while (mPleaseClose == PR_FALSE) {
    queue_fd = mEventQueue->GetEventQueueSelectFD();
    GrRegisterInput(queue_fd);

    if (CallTimeToNextTimeoutFunc(&cur_time) == 0) {
      GrGetNextEventTimeout(&event, 100);
      if (event.type == GR_EVENT_TYPE_TIMEOUT)
        continue;
    } else {
      if ((cur_time.tv_sec == 0) && (cur_time.tv_usec == 0)) {
        event.type = GR_EVENT_TYPE_TIMEOUT;
      } else {
        GR_TIMEOUT timeout = cur_time.tv_usec / 10000 + 1;
        GrGetNextEventTimeout(&event, timeout);
      }
    }
    if (event.type == GR_EVENT_TYPE_FDINPUT) {
      mEventQueue->ProcessPendingEvents();
    } else {
      nsAppShell::DispatchXEvent(&event);
    }

    if (mModalEventQueue) mModalEventQueue->ProcessPendingEvents();

    if (GrPeekEvent(&event) == 0) {
      nsWindow::UpdateIdle();
      GrFlush();
    }
  }

  mPleaseClose = PR_FALSE;
  Spindown();
  return NS_OK;
}

NS_IMETHODIMP nsAppShell::Spindown()
{
  if (mEventQueue) {
    mEventQueue->ProcessPendingEvents();
    mEventQueue = nsnull;
  }
  if (mModalEventQueue) {
    mModalEventQueue->ProcessPendingEvents();
    mModalEventQueue = nsnull;
  }

  return NS_OK;
}

NS_IMETHODIMP nsAppShell::Exit()
{
  mPleaseClose = PR_TRUE;
  return NS_OK;
}

NS_IMETHODIMP nsAppShell::GetNativeEvent(PRBool &aRealEvent, void *& aEvent)
{
  if (mEventQueue == nsnull)
    return NS_ERROR_NOT_INITIALIZED;

  GR_EVENT *event;

  GrRegisterInput(0);

  aRealEvent = PR_FALSE;
  event = (GR_EVENT *)malloc(sizeof(GR_EVENT));
  GrGetNextEventTimeout(event, 100);
  if (event->type != GR_EVENT_TYPE_TIMEOUT) {
    aRealEvent = PR_TRUE;
  }
  aEvent = event;
  return NS_OK;
}

NS_IMETHODIMP nsAppShell::DispatchNativeEvent(PRBool aRealEvent, void *aEvent)
{
  GR_EVENT *event;

  if (mModalEventQueue)
    mModalEventQueue->ProcessPendingEvents();
  else
    mEventQueue->ProcessPendingEvents();

  event = (GR_EVENT *)aEvent;

  if (aRealEvent) {
    if (event->type != GR_EVENT_TYPE_FDINPUT) {
      nsAppShell::DispatchXEvent(event);
    }
  }

  free(event);
  CallProcessTimeoutsProc();
  nsWindow::UpdateIdle();
  GrFlush();
  return NS_OK;
}

void nsAppShell::DispatchXEvent(GR_EVENT *event)
{
  switch (event->type) {
  case GR_EVENT_TYPE_EXPOSURE:
    HandleExposureEvent(event);
    break;
  case GR_EVENT_TYPE_BUTTON_DOWN:
    HandleFocusInEvent(event);
    HandleButtonEvent(event);
    break;
  case GR_EVENT_TYPE_BUTTON_UP:
    HandleButtonEvent(event);
    break;
  case GR_EVENT_TYPE_MOUSE_ENTER:
  	HandleFocusInEvent(event);
    HandleEnterEvent(event);
    break;
  case GR_EVENT_TYPE_FOCUS_IN:
    HandleFocusInEvent(event);
    break;
  case GR_EVENT_TYPE_MOUSE_EXIT:
    HandleLeaveEvent(event);
    break;
  case GR_EVENT_TYPE_FOCUS_OUT:
    HandleFocusOutEvent(event);
    break;
  case GR_EVENT_TYPE_MOUSE_MOTION:
    break;
  case GR_EVENT_TYPE_MOUSE_POSITION:
    HandleMouseEvent(event);
    break;
  case GR_EVENT_TYPE_KEY_DOWN:
    HandleKeyDownEvent(event);
    break;
  case GR_EVENT_TYPE_KEY_UP:
    HandleKeyUpEvent(event);
    break;
  case GR_EVENT_TYPE_UPDATE:
  case GR_EVENT_TYPE_CHLD_UPDATE:
    HandleUpdateEvent(event);
    break;
  case GR_EVENT_TYPE_CLOSE_REQ:
    HandleCloseEvent(event);
    break;
  case GR_EVENT_TYPE_ERROR:
#ifdef DEBUG
    fprintf(stderr, "Nano-X returned an error event\n");
#endif
    break;
  case GR_EVENT_TYPE_NONE:
#ifdef DEBUG_peter
   fprintf(stderr,"Nano-X return a NONE event\n");
#endif
    break;
  case GR_EVENT_TYPE_TIMEOUT:
    CallProcessTimeoutsProc();
    break;
  default:
    break;
  }

}

NS_IMETHODIMP nsAppShell::ListenToEventQueue(nsIEventQueue *aQueue, PRBool what)
{
  if (what) {
    if (mModalEventQueue != nsnull) {
      ListenToEventQueue(mModalEventQueue, PR_FALSE);
    }
    mModalEventQueue = aQueue;
  } else {
    if (aQueue) {
      aQueue->ProcessPendingEvents();
      if (aQueue == mModalEventQueue) {
        mModalEventQueue->ProcessPendingEvents();
        mModalEventQueue = nsnull;
      }
    }
  }
  return NS_OK;
}

void nsAppShell::HandleExposureEvent(GR_EVENT *event)
{
  nsPaintEvent pevent;
  nsWidget *widget = nsnull;

#ifdef DEBUG_peter
  fprintf(stderr, "HandleExposureEvent\n");
#endif

  widget = nsWidget::GetWidgetForWindow(event->exposure.wid);

  if (!widget)
    return;

  pevent.message = NS_PAINT;
  pevent.widget = widget;
  pevent.eventStructType = NS_PAINT_EVENT;
  pevent.rect = new nsRect();
  
  pevent.rect->SetRect(event->exposure.x, event->exposure.y,
                      event->exposure.width+1, event->exposure.height+1);
  pevent.time = PR_Now();
  NS_IF_ADDREF(widget);
  widget->OnPaint(pevent);
  NS_IF_RELEASE(widget);
  delete pevent.rect;
#ifdef DEBUG_peter
  fprintf(stderr,"/HandleExposureEvent\n");
#endif
}

void nsAppShell::HandleButtonEvent(GR_EVENT *event)
{
  nsWidget *widget = nsnull;
  nsMouseEvent mevent;
  PRUint32 eventType = 0;
  PRBool currentDragState = mDragging;

  mevent.isShift = mShiftDown;
  mevent.isControl = mCtrlDown;
  mevent.isAlt = mevent.isMeta = mAltDown;

  widget = nsWidget::GetWidgetForWindow(event->button.wid);
  if(!widget) {
  #ifdef DEBUG_mattw
  	NS_ASSERTION(widget != nsnull, "Hmm.. wrong even sent to this method?");
  #endif
  	return;
  }

#ifdef DEBUG_peter
  fprintf(stderr, "HandleButtonEvent\n");
#endif

	switch(event->type) {
		case GR_EVENT_TYPE_BUTTON_DOWN:
			if(event->button.changebuttons & GR_BUTTON_L) {
				mDragging = PR_TRUE;
				eventType = NS_MOUSE_LEFT_BUTTON_DOWN;
			} else if(event->button.changebuttons & GR_BUTTON_M) {
				eventType = NS_MOUSE_MIDDLE_BUTTON_DOWN;
			} else if(event->button.changebuttons & GR_BUTTON_R) {
				/* XXX look back into this in case anything
				 * actually needs a NS_MOUSE_RIGHT_BUTTON_DOWN
				 */ 
				 eventType = NS_CONTEXTMENU;
			}
			#ifdef DEBUG_mattw
				else printf("WaRnInG: Invalid mouse button Down: 0x%x\n",
						event->button.changebuttons);
			#endif
			
			break;
		case GR_EVENT_TYPE_BUTTON_UP:
			if(event->button.changebuttons & GR_BUTTON_L) {
				mDragging = PR_FALSE;
				eventType = NS_MOUSE_LEFT_BUTTON_UP;
			} else if(event->button.changebuttons & GR_BUTTON_M) {
				eventType = NS_MOUSE_MIDDLE_BUTTON_UP;
			} else if(event->button.changebuttons & GR_BUTTON_R) {
				 eventType = NS_MOUSE_RIGHT_BUTTON_UP;
			}
			#ifdef DEBUG_mattw
				else printf("WaRnInG: Invalid mouse button: 0x%x\n",
						event->button.changebuttons);
			#endif
			break;
	}

	mevent.widget = widget;
	mevent.point.x = event->button.x;
	mevent.point.y = event->button.y;
	mevent.time = PR_Now();

	if (PR_Now() - mClickTime > 1000000)
		mClicked = PR_FALSE;

	if (event->type == GR_EVENT_TYPE_BUTTON_DOWN) {
		if (!mClicked) {
			mClicked = PR_TRUE;
			mClickTime = PR_Now();
			mClicks = 1;
			mClickedButton = event->button.changebuttons;
		} else {
			mClickTime = PR_Now() - mClickTime;
			if ((mClickTime < 500000) && (mClickedButton == event->button.buttons))
				mClicks = 2;
			else
				mClicks = 1;
			mClicked = PR_FALSE;
		}
	}

	if(currentDragState && !mDragging)
		HandleDragDropEvent(event);

      // make sure this is in focus. This will do until I rewrite all the 
  // focus routines. KenF
  /* FIXME : Changed to PR_FALSE so when you enter you don't get focus */
  widget->SetFocus(PR_FALSE);



	mevent.message = eventType;
	mevent.eventStructType = NS_MOUSE_EVENT;
	mevent.clickCount = mClicks;
	NS_IF_ADDREF(widget);
	widget->DispatchMouseEvent(mevent);
	NS_IF_RELEASE(widget);
}

void nsAppShell::HandleEnterEvent(GR_EVENT *event)
{
#ifdef DEBUG_peter
  fprintf(stderr, "HandleEnterEvent\n");
#endif

  if (mDragging) {
    HandleDragEnterEvent(event);
  }

  nsMouseEvent enterEvent;
  nsWidget *widget = nsnull;

  widget = nsWidget::GetWidgetForWindow(event->general.wid);

  if (!widget) {
    NS_ASSERTION(widget != 0, "HandleEnter: Could not find widget for window");
    return;
  }

  enterEvent.widget = widget;
  enterEvent.time = PR_Now();
  enterEvent.point.x = 0;
  enterEvent.point.y = 0;
  enterEvent.message = NS_MOUSE_ENTER;
  enterEvent.eventStructType = NS_MOUSE_EVENT;

  NS_IF_ADDREF(widget);
  widget->DispatchWindowEvent(enterEvent);
  NS_IF_RELEASE(widget);
}

void nsAppShell::HandleLeaveEvent(GR_EVENT *event)
{
#ifdef DEBUG_peter
  fprintf(stderr, "HandleLeaveEvent\n");
#endif

  nsMouseEvent leaveEvent;
  nsWidget *widget = nsnull;

  if (mDragging) {
    HandleDragLeaveEvent(event);
  }

  widget = nsWidget::GetWidgetForWindow(event->general.wid);

  if (!widget)
    return;

  leaveEvent.widget = widget;
  leaveEvent.time = PR_Now();
  leaveEvent.point.x = 0;
  leaveEvent.point.y = 0;
  leaveEvent.message = NS_MOUSE_EXIT;
  leaveEvent.eventStructType = NS_MOUSE_EVENT;

  NS_IF_ADDREF(widget);
  widget->DispatchWindowEvent(leaveEvent);
  NS_IF_RELEASE(widget);
}

void nsAppShell::HandleMouseEvent(GR_EVENT *event)
{
#ifdef DEBUG_peter
  fprintf(stderr, "HandleMouseEvent\n");
#endif

  if (mDragging) {
    HandleDragMotionEvent(event);
  }

  nsMouseEvent mevent;
  nsWidget *widget = nsnull;

  widget = nsWidget::GetWidgetForWindow(event->mouse.wid);

  if (!widget)
    return;

  mevent.widget = widget;
  mevent.time = PR_Now();
  mevent.point.x = event->mouse.x;
  mevent.point.y = event->mouse.y;
  mevent.message = NS_MOUSE_MOVE;
  mevent.eventStructType = NS_MOUSE_EVENT;

  NS_IF_ADDREF(widget);
  widget->DispatchMouseEvent(mevent);
  NS_IF_RELEASE(widget);
}

void nsAppShell::HandleKeyDownEvent(GR_EVENT *event)
{
  PRUint32 virtualKeyCode;
  PRUint32 printableChar;
  PRBool specialKey;

  GR_WINDOW_ID focusWindow = 0;

  UpdateKeyModifiers(event);

  nsWidget *focusWidget = nsnull;

  focusWindow = nsWidget::GetFocusWindow();
  if (focusWindow) 
    focusWidget = nsWidget::GetWidgetForWindow(focusWindow);

  if (focusWidget == nsnull) {
#ifdef DEBUG
    fprintf(stderr, "Could not obtain focus widget\n");
#endif
    return;
  }


  nsWidget *tlWidget = focusWidget->GetTopLevelWidget();
  nsWidget *modalWidget = focusWidget->GetModalWindow();
  if (modalWidget) {
    modalWidget = modalWidget->GetTopLevelWidget();
    if (modalWidget != tlWidget) {
#ifdef DEBUG
	  fprintf(stderr, "Couldnt find top level window\n");
#endif
      return;
    }
  }


  if (event->keystroke.ch == 0) {
#ifdef DEBUG_mattw
    fprintf(stderr, "NULL key given to nsAppShell::HandleKeyDownEvent()\n");
#endif
    return;
  }

#if 0
  if(nsKeyCode::KeyCodeIsModifier(event)) {
	fprintf(stderr, "Skipping sole modifier\n");
	return;
  }
#endif

  specialKey = ((event->keystroke.ch & 0xff00) != 0);
  virtualKeyCode = nsKeyCode::ConvertKeyToVirtualKey(event);
  printableChar = specialKey ? 0 : event->keystroke.ch;

  if(mCtrlDown && !specialKey)
  	printableChar += 0x60;		// XXX ?

  // Generate mozilla key events
  nsKeyEvent keyEvent;
  keyEvent.keyCode = virtualKeyCode;
  keyEvent.charCode = 0;
  keyEvent.time = PR_Now();
  keyEvent.isShift = mShiftDown;
  keyEvent.isControl = mCtrlDown;
  keyEvent.isAlt = mAltDown;
  keyEvent.isMeta = PR_FALSE;
  keyEvent.point.x = 0;
  keyEvent.point.y = 0;
  keyEvent.message = NS_KEY_DOWN;
  keyEvent.widget = focusWidget;
  keyEvent.eventStructType = NS_KEY_EVENT;
  focusWidget->DispatchKeyEvent(keyEvent);

  // Fire off second NS_KEY_PRESS event 
  nsKeyEvent keyPressEvent;
  keyPressEvent.keyCode = virtualKeyCode;
  keyPressEvent.charCode = printableChar;
  keyPressEvent.time = PR_Now();
  keyPressEvent.isShift = mShiftDown;
  keyPressEvent.isControl = mCtrlDown;
  keyPressEvent.isAlt = mAltDown;
  keyPressEvent.isMeta = PR_FALSE;
  keyPressEvent.point.x = 0;
  keyPressEvent.point.y = 0;
  keyPressEvent.message = NS_KEY_PRESS;
  keyPressEvent.widget = focusWidget;
  keyPressEvent.eventStructType = NS_KEY_EVENT;

  if (printableChar) {
    if (!mCtrlDown && !mAltDown)
      keyPressEvent.isShift = PR_FALSE;
  }

  focusWidget->DispatchKeyEvent(keyPressEvent);
}

void nsAppShell::HandleKeyUpEvent(GR_EVENT *event)
{
  PRUint32 virtualKeyCode;
  PRBool specialKey;

  UpdateKeyModifiers(event);

  nsWidget *widget = nsnull;

  widget = nsWidget::GetWidgetForWindow(event->keystroke.wid);

  if (widget == nsnull) {
#ifdef DEBUG_mattw
    fprintf(stderr, "Could not obtain focus widget\n");
#endif
    return;
  }

  nsWidget *tlWidget = widget->GetTopLevelWidget();
  nsWidget *modalWidget = widget->GetModalWindow();
  if (modalWidget) {
    modalWidget = modalWidget->GetTopLevelWidget();
    if (modalWidget != tlWidget) {
      return;
    }
  }

  if (event->keystroke.ch == 0)
    return;

  specialKey = ((event->keystroke.ch & 0xff00) != 0);

  virtualKeyCode = nsKeyCode::ConvertKeyToVirtualKey(event);


  nsKeyEvent keyEvent;

  keyEvent.keyCode = virtualKeyCode;
  keyEvent.charCode = 0;
  keyEvent.time = PR_Now();
  keyEvent.isShift = mShiftDown;
  keyEvent.isControl = mCtrlDown;
  keyEvent.isAlt = mAltDown;
  keyEvent.isMeta = mAltDown;
  keyEvent.point.x = 0;
  keyEvent.point.y = 0;
  keyEvent.message = NS_KEY_UP;
  keyEvent.widget = widget;
  keyEvent.eventStructType = NS_KEY_EVENT;

  NS_ADDREF(widget);
  widget->DispatchKeyEvent(keyEvent);
  NS_RELEASE(widget);
}



void nsAppShell::HandleUpdateEvent(GR_EVENT *event)
{
  nsWidget *widget;

  widget = nsWidget::GetWidgetForWindow(event->update.wid);

  if (!widget) {
    NS_ASSERTION(widget != 0, "HandleUpdateEvent: The widget is not found");
    return;
  }

#ifdef DEBUG_mattw
  if(event->update.utype == 3) {
	  printf("nsAppShell::HandleUpdateEvent: utype=%d, wid=%d, subwid=%d\n",
		  event->update.utype, event->update.wid, event->update.subwid);
	  printf("nsAppShell::HandleUpdateEvent: x=%d, y=%d, w=%d, h=%d\n",
		  event->update.x, event->update.y, event->update.width,
		  event->update.height);
  }
#endif

  switch (event->update.utype) {
    case GR_UPDATE_MAP:
      NS_ADDREF(widget);
      if (widget->GetWindowType() == eWindowType_popup) {
        GrRaiseWindow((GR_WINDOW_ID)widget->GetNativeData(NS_NATIVE_WINDOW));
      }
      NS_RELEASE(widget);
      break;
  }
}

void nsAppShell::HandleCloseEvent(GR_EVENT *event)
{
#if defined(DEBUG_peter) || defined(DEBUG_mattw)
  fprintf(stderr, "HandleCloseEvent\n");
#endif
  nsWidget *w = nsWidget::GetWidgetForWindow(event->general.wid);

  // XXX This works, but I havent checked if its leaky yet.. :/
  if(w) {
  	w->Show(PR_FALSE);
	w->Destroy();
  } else {
	NS_ASSERTION(w != 0, "Null window returned in HandleCloseEvent");
  }
}

void nsAppShell::HandleFocusInEvent(GR_EVENT *event)
{
  nsWidget *widget;

  widget = nsWidget::GetWidgetForWindow(event->general.wid);

  if (!widget) {
    NS_ASSERTION(widget != 0, "HandleFocusIn: Could not find widget for window");
    return;
  }

  nsGUIEvent focusEvent;

  focusEvent.message = NS_GOTFOCUS;
  focusEvent.widget = widget;
  focusEvent.eventStructType = NS_GUI_EVENT;
  focusEvent.time = 0;
  focusEvent.point.x = 0;
  focusEvent.point.y = 0;

  NS_ADDREF(widget);
  widget->DispatchWindowEvent(focusEvent);
  NS_RELEASE(widget);
}

void nsAppShell::HandleFocusOutEvent(GR_EVENT *event)
{
  nsWidget *widget;

  widget = nsWidget::GetWidgetForWindow(event->general.wid);

  if (!widget) {
    NS_ASSERTION(widget != 0, "HandleFocusOut: Could not find widget for window");
    return;
  }

  nsGUIEvent focusEvent;

  focusEvent.message = NS_LOSTFOCUS;
  focusEvent.widget = widget;
  focusEvent.eventStructType = NS_GUI_EVENT;
  focusEvent.time = PR_Now();
  focusEvent.point.x = 0;
  focusEvent.point.y = 0;

  NS_ADDREF(widget);
  widget->DispatchWindowEvent(focusEvent);
  NS_RELEASE(widget);
}

void nsAppShell::HandleDragMotionEvent(GR_EVENT *event)
{
  nsMouseEvent mEvent;
  PRBool currentDragState = PR_FALSE;
  nsresult rv;

  nsWidget *widget;

  widget = nsWidget::GetWidgetForWindow(event->mouse.subwid);

  nsCOMPtr<nsIDragService> dragService(do_GetService(kCDragServiceCID, &rv));;
  nsCOMPtr<nsIDragSessionNanoX> dragServiceNanoX;

  if (NS_SUCCEEDED(rv)) {
    dragServiceNanoX = do_QueryInterface(dragService);
    if (dragServiceNanoX) {
      dragServiceNanoX->IsDragging(&currentDragState);
    }
  }

  if (currentDragState) {
    dragServiceNanoX->UpdatePosition(event->mouse.rootx, event->mouse.rooty);

    if (!widget)
      return;

    mEvent.widget = widget;
    mEvent.point.x = event->mouse.x;
    mEvent.point.y = event->mouse.y;

    mEvent.message = NS_DRAGDROP_OVER;
    mEvent.eventStructType = NS_DRAGDROP_EVENT;
 
    NS_ADDREF(widget);
    widget->DispatchMouseEvent(mEvent);
    NS_RELEASE(widget);
  }
}

void nsAppShell::HandleDragEnterEvent(GR_EVENT *event)
{
  nsMouseEvent enterEvent;
  PRBool currentDragState = PR_FALSE;
  nsWidget *widget;
  nsresult rv;

  widget = nsWidget::GetWidgetForWindow(event->general.wid);

  if (!widget)
    return;

  nsCOMPtr<nsIDragService> dragService(do_GetService(kCDragServiceCID, &rv));;

  if (NS_SUCCEEDED(rv)) {
    nsCOMPtr<nsIDragSessionNanoX> dragServiceNanoX;
    dragServiceNanoX = do_QueryInterface(dragService);
    if (dragServiceNanoX) {
      dragServiceNanoX->IsDragging(&currentDragState);
    }
  }

  if (currentDragState) {
    enterEvent.widget = widget;
    enterEvent.point.x = 0;
    enterEvent.point.y = 0;

    enterEvent.message = NS_DRAGDROP_ENTER;
    enterEvent.eventStructType = NS_DRAGDROP_EVENT;

    NS_ADDREF(widget);
    widget->DispatchWindowEvent(enterEvent);
    NS_RELEASE(widget);
  }
}

void nsAppShell::HandleDragLeaveEvent(GR_EVENT *event)
{
  nsMouseEvent leaveEvent;
  PRBool currentDragState = PR_FALSE;
  nsWidget *widget;
  nsresult rv;

  widget = nsWidget::GetWidgetForWindow(event->general.wid);

  if (!widget)
    return;

  nsCOMPtr<nsIDragService> dragService(do_GetService(kCDragServiceCID, &rv));;

  // FIXME: Not sure if currentlyDragging is required. KenF
  if (NS_SUCCEEDED(rv)) {
    nsCOMPtr<nsIDragSessionNanoX> dragServiceNanoX;
    dragServiceNanoX = do_QueryInterface(dragService);
    if (dragServiceNanoX) {
      dragServiceNanoX->IsDragging(&currentDragState);
    }
  }

  if (currentDragState) {
    leaveEvent.widget = widget;
    leaveEvent.message = NS_DRAGDROP_EXIT;
    leaveEvent.eventStructType = NS_DRAGDROP_EVENT;

    NS_ADDREF(widget);
    widget->DispatchWindowEvent(leaveEvent);
    NS_RELEASE(widget);
  }
}

void nsAppShell::HandleDragDropEvent(GR_EVENT *event)
{
  nsMouseEvent mEvent;
  PRBool currentDragState = PR_FALSE;
  nsWidget *widget;
  nsresult rv;

  widget = nsWidget::GetWidgetForWindow(event->button.subwid);

  if (!widget)
    return;

  nsCOMPtr<nsIDragService> dragService(do_GetService(kCDragServiceCID, &rv));;

  // FIXME: Dont think the currentlyDragging check is required. KenF
  if (NS_SUCCEEDED(rv)) {
    nsCOMPtr<nsIDragSessionNanoX> dragServiceNanoX;
    dragServiceNanoX = do_QueryInterface(dragService);
    if (dragServiceNanoX) {
      dragServiceNanoX->IsDragging(&currentDragState);
    }
  }

  if (currentDragState) {
    mEvent.widget = widget;
    mEvent.point.x = 0;
    mEvent.point.y = 0;

    mEvent.message = NS_DRAGDROP_DROP;
    mEvent.eventStructType = NS_DRAGDROP_EVENT;

    NS_ADDREF(widget);
    widget->DispatchMouseEvent(mEvent);
    NS_RELEASE(widget);

    dragService->EndDragSession();
  }
}

void nsAppShell::UpdateKeyModifiers(GR_EVENT *e)
{

   if(e->type == GR_EVENT_TYPE_KEY_DOWN ||
       e->type == GR_EVENT_TYPE_KEY_UP) {
       mShiftDown = ((e->keystroke.modifiers & MWKMOD_SHIFT) != 0);
       mAltDown = ((e->keystroke.modifiers & MWKMOD_ALT) != 0); 
       mCtrlDown = ((e->keystroke.modifiers & MWKMOD_CTRL) != 0);
   }   
}  

