/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s):
 *   Peter Hartshorn <peter@igelaus.com.au>
 *   B.J. Rossiter <bj@igelaus.com.au>
 */

#ifndef nsAppShell_h__
#define nsAppShell_h__

#include "nsIAppShell.h"
#include "nsIEventQueue.h"
#include "nsIEventQueueService.h"
#include "prtime.h"
#include <nano-X.h>

/**
 * Native nano-X Application shell wrapper
 */
class nsAppShell : public nsIAppShell
{
public:
  nsAppShell();
  virtual ~nsAppShell();

  NS_DECL_ISUPPORTS
  NS_DECL_NSIAPPSHELL

private:
  nsDispatchListener* mDispatchListener;
  static void HandleButtonEvent(GR_EVENT *event);
  static void HandleExposureEvent(GR_EVENT *event);
  static void HandleDragMotionEvent(GR_EVENT *event);
  static void HandleEnterEvent(GR_EVENT *event);
  static void HandleLeaveEvent(GR_EVENT *event);
  static void HandleMouseEvent(GR_EVENT *event);
  static void HandleKeyUpEvent(GR_EVENT *event);
  static void HandleKeyDownEvent(GR_EVENT *event);
  static void HandleUpdateEvent(GR_EVENT *event);
  static void HandleCloseEvent(GR_EVENT *event);
  static void HandleFocusInEvent(GR_EVENT *event);
  static void HandleFocusOutEvent(GR_EVENT *event);
  static void HandleDragEnterEvent(GR_EVENT *event);
  static void HandleDragLeaveEvent(GR_EVENT *event);
  static void HandleDragDropEvent(GR_EVENT *event);

  static void DispatchXEvent(GR_EVENT *event);
  static PRInt32 mButton;
  static PRBool mDragging;
  static PRBool mIsInitialized;
  static PRUint32 mAppShellCount;
  nsCOMPtr<nsIEventQueue> mEventQueue;
  nsCOMPtr<nsIEventQueue> mModalEventQueue;
  static PRBool mPleaseClose;
  static PRUint32 escapeSequenceCount;
  static PRUint32 storedSpecialKey;

  static PRBool mClicked;
  static PRTime mClickTime;
  static PRInt8 mClicks;
  static PRUint16 mClickedButton;

  // update the following 3 flags
  static void UpdateKeyModifiers(GR_EVENT *e);
  static PRBool mShiftDown;
  static PRBool mCtrlDown;
  static PRBool mAltDown;

  nsIEventQueueService *mEventQueueService;
 //nsIEventQueue *mEventQueue;

};

#endif /* nsAppShell_h__ */
