/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s):
 *   Peter Hartshorn <peter@igelaus.com.au>
 *   B.J. Rossiter <bj@igelaus.com.au>
 */

/* TODO:
 * Currently this only supports the transfer of TEXT! FIXME
 */

#include "nsClipboard.h"

#include <nano-X.h>
#include "string.h"

#include "nsCOMPtr.h"
#include "nsFileSpec.h"
#include "nsCRT.h"
#include "nsISupportsArray.h"
#include "nsISupportsPrimitives.h"

#include "nsIComponentManager.h"
#include "nsIServiceManager.h"
#include "nsWidgetsCID.h"
#include "nsXPIDLString.h"
#include "nsPrimitiveHelpers.h"

#include "nsTextFormatter.h"
#include "nsVoidArray.h"

#include "nsIServiceManager.h"
#include "nsICharsetConverterManager.h"
// unicode conversion
#define NS_IMPL_IDS
#  include "nsIPlatformCharset.h"
#undef NS_IMPL_IDS


// The class statics:
nsCOMPtr<nsITransferable>   nsClipboard::mTransferable = nsnull;
GR_WINDOW_ID                      nsClipboard::sWindow;

#if defined(DEBUG_mcafee) || defined(DEBUG_pavlov)
#define DEBUG_CLIPBOARD
#endif

NS_IMPL_ISUPPORTS1(nsClipboard, nsIClipboard);

nsClipboard::nsClipboard()
{
  NS_INIT_REFCNT();
  Init();
}

nsClipboard::~nsClipboard()
{
}

// Initialize the clipboard and create a nsWidget for communications

void nsClipboard::Init()
{
  sWidget = new nsWidget();
  const nsRect rect(0,0,100,100);

  sWidget->Create((nsIWidget *)nsnull, rect, Callback,
                  (nsIDeviceContext *)nsnull, (nsIAppShell *)nsnull,
                  (nsIToolkit *)nsnull, (nsWidgetInitData *)nsnull);
  sWindow = (GR_WINDOW_ID)sWidget->GetNativeData(NS_NATIVE_WINDOW);

}

nsEventStatus PR_CALLBACK nsClipboard::Callback(nsGUIEvent *event)
{
  if (mTransferable == nsnull) {
#ifdef DEBUG
    fprintf(stderr, "nsClipboard::Callback: null transferable\n");
#endif
    return nsEventStatus_eIgnore;
  }

  // Get the data from the Transferable

  char *dataFlavor = kUnicodeMime;
  nsCOMPtr<nsISupports> genDataWrapper;
  nsresult rv;
  PRUint32 dataLength;
  void *data;
  data = malloc(16384);
  rv = mTransferable->GetTransferData(dataFlavor,
                                      getter_AddRefs(genDataWrapper),
                                      &dataLength);
  nsPrimitiveHelpers::CreateDataFromPrimitive(dataFlavor, genDataWrapper,
                                              &data, dataLength);
  if (NS_SUCCEEDED(rv) && data && dataLength) {
    char *plainText = nsnull;
    PRUnichar* unicodeData = NS_REINTERPRET_CAST(PRUnichar*, data);
    PRInt32 plainLen = 0;
    nsPrimitiveHelpers::ConvertUnicodeToPlatformPlainText(unicodeData,
                                                          dataLength/2,
                                                          &plainText,
                                                          &plainLen);
    if (data) {
      free(data);
      data = plainText;
      dataLength = plainLen;
    }
  }
  return nsEventStatus_eIgnore;
}

nsITransferable *nsClipboard::GetTransferable(PRInt32 aWhichClipboard)
{
  nsITransferable *transferable = nsnull;
  switch (aWhichClipboard)
  {
  case kGlobalClipboard:
    transferable = mGlobalTransferable;
    break;
  case kSelectionClipboard:
    transferable = mSelectionTransferable;
    break;
  }
  NS_IF_ADDREF(transferable);
  return transferable;
}

// Ripped from GTK. Does the writing to the appropriate transferable.
// Does *some* flavour stuff, but not all!!!
// FIXME: Needs to be completed.
NS_IMETHODIMP nsClipboard::SetNativeClipboardData(PRInt32 aWhichClipboard)
{
  // get flavor list that includes all flavors that can be written (including ones 
  // obtained through conversion)
  nsCOMPtr<nsISupportsArray> flavorList;
  nsCOMPtr<nsITransferable> transferable(getter_AddRefs(GetTransferable(aWhichClipboard)));

  // make sure we have a good transferable
  if (nsnull == transferable) {
    return NS_ERROR_FAILURE;
  }

  nsresult errCode = transferable->FlavorsTransferableCanExport ( getter_AddRefs(flavorList) );
  if ( NS_FAILED(errCode) )
    return NS_ERROR_FAILURE;

  PRUint32 cnt;
  flavorList->Count(&cnt);
  for ( PRUint32 i=0; i<cnt; ++i )
  {
    nsCOMPtr<nsISupports> genericFlavor;
    flavorList->GetElementAt ( i, getter_AddRefs(genericFlavor) );
    nsCOMPtr<nsISupportsString> currentFlavor ( do_QueryInterface(genericFlavor) );
    if ( currentFlavor ) {
      nsXPIDLCString flavorStr;
      currentFlavor->ToString(getter_Copies(flavorStr));
    }
  }

  mIgnoreEmptyNotification = PR_FALSE;
  return NS_OK;
}

NS_IMETHODIMP nsClipboard::SetData(nsITransferable *aTransferable,
                                   nsIClipboardOwner *anOwner,
                                   PRInt32 aWhichClipboard) 
{
  // Check from GTK. (must double check this!!)
  if ((aTransferable == mGlobalTransferable.get() &&
       anOwner == mGlobalOwner.get() &&
       aWhichClipboard == kGlobalClipboard ) ||
      (aTransferable == mSelectionTransferable.get() &&
       anOwner == mSelectionOwner.get() &&
       aWhichClipboard == kSelectionClipboard)
      )
    {
      return NS_OK;
    }
  
  EmptyClipboard(aWhichClipboard);

  switch(aWhichClipboard) {
  case kSelectionClipboard:
    mSelectionOwner = anOwner;
    mSelectionTransferable = aTransferable;
    break;
  case kGlobalClipboard:
    mGlobalOwner = anOwner;
    mGlobalTransferable = aTransferable;
    break;
  }
  
  return SetNativeClipboardData(aWhichClipboard);
}

NS_IMETHODIMP nsClipboard::GetData(nsITransferable *aTransferable,
                                   PRInt32 aWhichClipboard)
{
  if (aTransferable == nsnull) {
#ifdef DEBUG_mattw
    fprintf(stderr, "nsClipboard::GetData: NULL transferable\n");
#endif
    return NS_ERROR_FAILURE;
  }


  // Get which transferable we should use.
  mTransferable = GetTransferable(aWhichClipboard);

  if(mTransferable == nsnull) {
#ifdef DEBUG_mattw
	fprintf(stderr, "nsClipboard::GetData: NULL transferable\n");
#endif
	return NS_ERROR_FAILURE;
  }
  
  char *dataFlavor = kUnicodeMime;
  nsCOMPtr<nsISupports> genDataWrapper;
  nsresult rv;
  PRUint32 dataLength;
  nsCOMPtr<nsITransferable> trans = do_QueryInterface(aTransferable);
  if (!trans)
    return NS_ERROR_FAILURE;

  rv = mTransferable->GetTransferData(dataFlavor,
                                      getter_AddRefs(genDataWrapper),
                                      &dataLength);
  if (NS_SUCCEEDED(rv)) {
    rv = trans->SetTransferData(dataFlavor,
                                genDataWrapper,
                                dataLength);
  }
  return NS_OK;
}

NS_IMETHODIMP nsClipboard::EmptyClipboard(PRInt32 aWhichClipboard) 
{
  if (mIgnoreEmptyNotification) {
    return NS_OK;
  }

  switch(aWhichClipboard) {
  case kSelectionClipboard:
    if (mSelectionOwner) {
      mSelectionOwner->LosingOwnership(mSelectionTransferable);
      mSelectionOwner = nsnull;
    }
    mSelectionTransferable = nsnull;
    break;
  case kGlobalClipboard:
    if (mGlobalOwner) {
      mGlobalOwner->LosingOwnership(mGlobalTransferable);
      mGlobalOwner = nsnull;
    }
    mGlobalTransferable = nsnull;
    break;
  }
  return NS_OK;
}

NS_IMETHODIMP nsClipboard::ForceDataToClipboard(PRInt32 aWhichClipboard)
{
  return NS_OK;
}

NS_IMETHODIMP nsClipboard::HasDataMatchingFlavors(nsISupportsArray *aFlavorList,
                                                  PRInt32 aWhichClipboard,
                                                  PRBool *_retval)
{
  *_retval = PR_TRUE;
  return NS_OK;
}

NS_IMETHODIMP nsClipboard::SupportsSelectionClipboard(PRBool *_retval)
{
  *_retval = PR_TRUE;
  return NS_OK;
}
