/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s):
 *   Peter Hartshorn <peter@igelaus.com.au>
 *   B.J. Rossiter <bj@igelaus.com.au>
 */

#include "nsDragService.h"
#include "nsWidgetsCID.h"
#include "nsIWidget.h"
#include "nsIServiceManager.h"
#include "nsVoidArray.h"
#include "nsXPIDLString.h"
#include "nsISupportsPrimitives.h"
#include "nsPrimitiveHelpers.h"
#include "nsString.h"

NS_IMPL_ADDREF_INHERITED(nsDragService, nsBaseDragService)
NS_IMPL_RELEASE_INHERITED(nsDragService, nsBaseDragService)
NS_IMPL_QUERY_INTERFACE3(nsDragService, nsIDragService, nsIDragSession, nsIDragSessionXlib)

nsWidget *nsDragService::sWidget = nsnull;
GR_WINDOW_ID    nsDragService::sWindow;
PRBool    nsDragService::mDragging = PR_FALSE;


#undef DEBUG_peter
nsDragService::nsDragService()
{
  mCanDrop = PR_FALSE;
  sWindow = 0;
}


nsDragService::~nsDragService()
{
}

// nsIDragService
NS_IMETHODIMP nsDragService::InvokeDragSession (nsIDOMNode *aDOMNode,
                                                nsISupportsArray * anArrayTransferables,
                                                nsIScriptableRegion * aRegion,
                                                PRUint32 aActionType)
{
  nsresult rv;
  PRUint32 numItemsToDrag = 0;
  GR_WM_PROPERTIES prop;

  nsBaseDragService::InvokeDragSession(aDOMNode, anArrayTransferables,
                                       aRegion, aActionType);

  if (!anArrayTransferables)
    return NS_ERROR_INVALID_ARG;

  mDataItems = anArrayTransferables;
  NS_ADDREF(mDataItems);

  rv = mDataItems->Count(&numItemsToDrag);
#ifdef DEBUG
fprintf(stderr, "nsDragService::InvokeDragSession %d\n", numItemsToDrag);
#endif
  if (!numItemsToDrag)
    return NS_ERROR_FAILURE;

  if (numItemsToDrag > 1) {
#ifdef DEBUG
    fprintf(stderr, "nsDragService: Cannot drag more than one item!\n");
#endif
   
    return NS_ERROR_FAILURE;
  }
  mDragging = PR_TRUE;

  if (sWindow == 0) {
    sWindow = GrNewWindow(GR_ROOT_WINDOW_ID, 0,0,16,16,0,0,0);
    prop.props = GR_WM_PROPS_NODECORATE | GR_WM_PROPS_NOAUTOMOVE |
                 GR_WM_PROPS_NOFOCUS | GR_WM_PROPS_NOBACKGROUND;
    prop.flags = GR_WM_FLAGS_PROPS;

    GrSetWMProperties(sWindow, &prop);
    GrSelectEvents(sWindow, GR_EVENT_MASK_ALL);

    GrMapWindow(sWindow);
  }
  return NS_OK;
}




NS_IMETHODIMP nsDragService::StartDragSession()
{
  mDragging = PR_TRUE;
  return nsBaseDragService::StartDragSession();
}

NS_IMETHODIMP nsDragService::EndDragSession()
{
  if (sWindow) {
    GrDestroyWindow(sWindow);
    sWindow = 0;
  }
  mDragging = PR_FALSE;
  return nsBaseDragService::EndDragSession();
}


// nsIDragSession

// For some reason we need this, but GTK does not. Hmmm...
NS_IMETHODIMP nsDragService::GetCurrentSession (nsIDragSession **aSession)
{
  if (!aSession)
    return NS_ERROR_FAILURE;

  if (!mDragging) {
    *aSession = nsnull;
    return NS_OK;
  }

  *aSession = (nsIDragSession *)this;
  NS_ADDREF(*aSession);
  return NS_OK;
}

NS_IMETHODIMP nsDragService::SetCanDrop(PRBool           aCanDrop)
{
  mCanDrop = aCanDrop;
  return NS_OK;
}

NS_IMETHODIMP nsDragService::GetCanDrop(PRBool          *aCanDrop)
{
#ifdef DEBUG
fprintf(stderr, "GetCanDrop %d\n", mCanDrop);
#endif
  *aCanDrop = mCanDrop;
  return NS_OK;
}

NS_IMETHODIMP nsDragService::GetNumDropItems(PRUint32 * aNumItems)
{
  // FIXME: Currently we can only drop one item, cos we are lame, just like gtk
  *aNumItems = 1;
  return NS_OK;
}

NS_IMETHODIMP nsDragService::GetData(nsITransferable * aTransferable, PRUint32 anItemIndex)
{
  nsresult rv;
  nsCOMPtr<nsISupportsArray> flavourList;

  rv = aTransferable->FlavorsTransferableCanImport(getter_AddRefs(flavourList));

  if (NS_FAILED(rv))
    return rv;

  nsCOMPtr<nsISupports> genericWrapper;
  flavourList->GetElementAt(0, getter_AddRefs(genericWrapper));
  nsCOMPtr<nsISupportsString> currentFlavour;
  currentFlavour = do_QueryInterface(genericWrapper);
  if (currentFlavour) {
    nsXPIDLCString flavourStr;
    currentFlavour->ToString(getter_Copies(flavourStr));

    nsCOMPtr<nsISupports> genItem;
    mDataItems->GetElementAt(0, getter_AddRefs(genItem));

    nsCOMPtr<nsITransferable> item (do_QueryInterface(genItem));
    nsCOMPtr<nsISupports> data;
    PRUint32 dataLen = 0;

    item->GetTransferData(flavourStr, getter_AddRefs(data), &dataLen);
    if (dataLen == 0) {
      data = nsnull;
      item->GetTransferData("text/unicode", getter_AddRefs(data), &dataLen);
      //flavourStr = "text/unicode";
      flavourStr.Adopt("text/unicode");
    }
    aTransferable->SetTransferData(flavourStr, data, dataLen);
  }

  EndDragSession();

  return NS_OK;
}

NS_IMETHODIMP nsDragService::IsDataFlavorSupported (const char *aDataFlavor, PRBool *_retval)
{
  //FIXME: we are lame

  const char *name = "text/unicode";

  if (strcmp(name, aDataFlavor) == 0) {
    *_retval = PR_TRUE;
  } else {
    *_retval = PR_FALSE;
  }
  return NS_OK;
}

//nsIDragSessionNanoX
NS_IMETHODIMP nsDragService::IsDragging(PRBool *result) {
  *result = mDragging;
  return NS_OK;
}

NS_IMETHODIMP nsDragService::UpdatePosition(PRInt32 x, PRInt32 y){
  GR_GC_ID aGC;
  
  if (sWindow) {
    aGC = GrNewGC();
    GrMoveWindow(sWindow, x+16, y+16);
    GrSetGCForeground(aGC, 0xffffff);
    GrFillRect(sWindow, aGC, 0,0,16,16);
    GrSetGCForeground(aGC, 0xff0000);
    GrRect(sWindow, aGC, 0, 0, 15, 15);
    GrLine(sWindow, aGC, 2, 2, 12, 2);
    GrLine(sWindow, aGC, 2, 4, 12, 4);
    GrLine(sWindow, aGC, 2, 6, 12, 6);
    GrLine(sWindow, aGC, 2, 8, 12, 8);
    GrLine(sWindow, aGC, 2, 10, 12, 10);
    GrLine(sWindow, aGC, 2, 12, 12, 12);
    GrDestroyGC(aGC);
  }
  return NS_OK;
}
