/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s):
 *   Peter Hartshorn <peter@igelaus.com.au>
 *   B.J. Rossiter <bj@igelaus.com.au>
 */

#ifndef nsWidget_h__
#define nsWidget_h__

#include <nsBaseWidget.h>
#include <nsList.h>
#include <nsIRegion.h>
#include <nsHashtable.h>
#include <nsIRollupListener.h>
#include "nsINanoXWindowService.h"
#include <nano-X.h>

class nsWidget : public nsBaseWidget
{
public:
  nsWidget();
  virtual ~nsWidget();

  NS_IMETHOD Create(nsIWidget *aParent,
                    const nsRect &aRect,
                    EVENT_CALLBACK aHandleEventFunction,
                    nsIDeviceContext *aContext,
                    nsIAppShell *aAppShell = nsnull,
                    nsIToolkit *aToolkit = nsnull,
                    nsWidgetInitData *aInitData = nsnull);

  NS_IMETHOD Create(nsNativeWidget aParent,
                    const nsRect &aRect,
                    EVENT_CALLBACK aHandleEventFunction,
                    nsIDeviceContext *aContext,
                    nsIAppShell *aAppShell = nsnull,
                    nsIToolkit *aToolkit = nsnull,
                    nsWidgetInitData *aInitData = nsnull);

  virtual nsresult CommonCreate(nsIWidget *aParent,
                                const nsRect &aRect,
                                EVENT_CALLBACK aHandleEventFunction,
                                nsIDeviceContext *aContext,
                                nsIAppShell *aAppShell,
                                nsIToolkit *aToolkit,
                                nsWidgetInitData *aInitData,
                                nsNativeWidget aNativeParent = nsnull);
  NS_IMETHOD Destroy();
  virtual nsIWidget *GetParent(void);
  NS_IMETHOD SetModal(PRBool aModal);
  NS_IMETHOD Show(PRBool bState);
  NS_IMETHOD IsVisible(PRBool &aState);

  NS_IMETHOD ConstrainPosition(PRInt32 *aX, PRInt32 *aY);
  NS_IMETHOD Move(PRInt32 aX, PRInt32 aY);
  NS_IMETHOD Resize(PRInt32 aWidth,
                    PRInt32 aHeight,
                    PRBool   aRepaint);
  NS_IMETHOD Resize(PRInt32 aX,
                    PRInt32 aY,
                    PRInt32 aWidth,
                    PRInt32 aHeight,
                    PRBool   aRepaint);

  NS_IMETHOD Enable(PRBool bState);
  NS_IMETHOD SetFocus(PRBool aRaise);
  NS_IMETHOD SetName(const char * aName);
  NS_IMETHOD SetBackgroundColor(const nscolor &aColor);
  virtual nsIFontMetrics* GetFont(void);
  NS_IMETHOD SetFont(const nsFont &aFont);
  NS_IMETHOD SetCursor(nsCursor aCursor);
  void LockCursor(PRBool aLock);

  NS_IMETHOD Invalidate(PRBool aIsSynchronous);
  NS_IMETHOD Invalidate(const nsRect & aRect,
                        PRBool aIsSynchronous);
  NS_IMETHOD Update();
  virtual void* GetNativeData(PRUint32 aDataType);
  NS_IMETHOD SetColorMap(nsColorMap *aColorMap);
  NS_IMETHOD Scroll(PRInt32 aDx, PRInt32 aDy, nsRect *aClipRect);
  NS_IMETHOD SetMenuBar(nsIMenuBar * aMenuBar);
  NS_IMETHOD ShowMenuBar(PRBool aShow);
  NS_IMETHOD SetTooltips(PRUint32 aNumberOfTips,nsRect* aTooltipAreas[]);
  NS_IMETHOD RemoveTooltips();
  NS_IMETHOD UpdateTooltips(nsRect* aNewTips[]);
  NS_IMETHOD WidgetToScreen(const nsRect& aOldRect, nsRect& aNewRect);
  NS_IMETHOD ScreenToWidget(const nsRect& aOldRect, nsRect& aNewRect);
  NS_IMETHOD BeginResizingChildren(void);
  NS_IMETHOD EndResizingChildren(void);
  NS_IMETHOD GetPreferredSize(PRInt32& aWidth, PRInt32& aHeight);
  NS_IMETHOD SetPreferredSize(PRInt32 aWidth, PRInt32 aHeight);
  NS_IMETHOD DispatchEvent(nsGUIEvent* event, nsEventStatus & aStatus);
  NS_IMETHOD SetBounds(const nsRect &aRect);
  NS_IMETHOD GetRequestedBounds(nsRect &aRect);

  NS_IMETHOD CaptureRollupEvents(nsIRollupListener *aListener,
                                 PRBool aDoCapture, PRBool aConsumeRollupEvent);
  NS_IMETHOD SetTitle(const nsString& title);

  nsWidget *GetTopLevelWidget(void);
  nsWidget *GetModalWindow(void);
  nsWindowType GetWindowType(void);

  static GR_WINDOW_ID GetFocusWindow(void);

  virtual PRBool DispatchWindowEvent(nsGUIEvent &aEvent);
  virtual PRBool DispatchMouseEvent(nsMouseEvent &aEvent);
  virtual PRBool DispatchKeyEvent(nsKeyEvent &aKeyEvent);
  virtual PRBool DispatchDestroyEvent(void);

  virtual PRBool OnPaint(nsPaintEvent &event);
  virtual PRBool OnResize(nsSizeEvent &event);
  virtual PRBool OnDeleteWindow(void);

  virtual void OnDestroy(void);

  static nsWidget *GetWidgetForWindow(GR_WINDOW_ID win);
protected:

  virtual void DestroyNative(void);

  void WidgetMove       (nsWidget *aWidget);
  void WidgetMoveResize (nsWidget *aWidget);
  void WidgetResize     (nsWidget *aWidget);
  void WidgetShow       (nsWidget *aWidget);

  void Map();
  void Unmap();

  PRBool ConvertStatus(nsEventStatus aStatus);

  static void AddWindowCallback(GR_WINDOW_ID aWindow, nsWidget *aWidget);
  static void DeleteWindowCallback(GR_WINDOW_ID aWindow);

  nsString mName;
  GR_WINDOW_ID mBaseWindow;
  GR_WINDOW_ID mParentWindow;
  static GR_WINDOW_ID mFocusWindow;
  nsIWidget *mParentWidget;
  PRBool mListenForResizes;
  PRBool mIsToplevel;
  PRBool mIsShown;

  nsRect mRequestedSize;

  nsCOMPtr<nsIRegion> mUpdateArea;

  
  static nsHashtable *gsWindowList;
  static nsList *gsModalList;

  // Variables for infomation about the current popup window and its listener
  static nsCOMPtr<nsIRollupListener> gRollupListener;
  static nsCOMPtr<nsIWidget> gRollupWidget;
  static PRBool   gRollupConsumeRollupEvent;

  PRBool IsMouseInWindow(nsIWidget* inWidget, PRInt32 inMouseX, PRInt32 inMouseY);
  PRBool HandlePopup( PRInt32 inMouseX, PRInt32 inMouseY);

  static nsNanoXWindowCallback	gsWindowCreateCallback;
  static nsNanoXWindowCallback	gsWindowDestroyCallback;
  static nsNanoXEventDispatcher	gsEventDispatcher;

  static void LoadCursor(const unsigned char *bits, const int size, MWIMAGEBITS *out);
};

#endif 
