/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s): 
 *   Peter Hartshorn <peter@igelaus.com.au>
 *   Ken Faulkner <faulkner@igelaus.com.au>
 */

#include "nsIGenericFactory.h"
#include "nsIModule.h"
#include "nsCOMPtr.h"
#include "nsWidgetsCID.h"

#include "nsWindow.h"
#include "nsButton.h"
#include "nsCheckButton.h"
#include "nsFileWidget.h"
#include "nsTextWidget.h"
#include "nsAppShell.h"
#include "nsToolkit.h"
#include "nsLookAndFeel.h"
#include "nsLabel.h"
#include "nsTransferable.h"
#include "nsClipboard.h"
#include "nsHTMLFormatConverter.h"
//#include "nsFontRetrieverService.h"
#include "nsDragService.h"
#include "nsFileSpecWithUIImpl.h"
#include "nsScrollBar.h"
#include "nsSound.h"

#ifdef IBMBIDI
#include "nsBidiKeyboard.h"
#endif


NS_GENERIC_FACTORY_CONSTRUCTOR(nsWindow)
NS_GENERIC_FACTORY_CONSTRUCTOR(ChildWindow)
//NS_GENERIC_FACTORY_CONSTRUCTOR(nsButton)
//NS_GENERIC_FACTORY_CONSTRUCTOR(nsCheckButton)
//NS_GENERIC_FACTORY_CONSTRUCTOR(nsFileWidget)
//NS_GENERIC_FACTORY_CONSTRUCTOR(nsTextWidget)
NS_GENERIC_FACTORY_CONSTRUCTOR(nsAppShell)
NS_GENERIC_FACTORY_CONSTRUCTOR(nsToolkit)
NS_GENERIC_FACTORY_CONSTRUCTOR(nsLookAndFeel)
//NS_GENERIC_FACTORY_CONSTRUCTOR(nsLabel)
NS_GENERIC_FACTORY_CONSTRUCTOR(nsTransferable)
NS_GENERIC_FACTORY_CONSTRUCTOR(nsClipboard)
NS_GENERIC_FACTORY_CONSTRUCTOR(nsHTMLFormatConverter)
//NS_GENERIC_FACTORY_CONSTRUCTOR(nsFontRetrieverService)
NS_GENERIC_FACTORY_CONSTRUCTOR(nsDragService)
NS_GENERIC_FACTORY_CONSTRUCTOR(nsFileSpecWithUIImpl)
NS_GENERIC_FACTORY_CONSTRUCTOR(nsSound)
#ifdef IBMBIDI
NS_GENERIC_FACTORY_CONSTRUCTOR(nsBidiKeyboard)
#endif

static nsresult nsHorizScrollbarConstructor (nsISupports *aOuter, REFNSIID aIID, void **aResult)
{
  nsresult rv;
  nsISupports *inst = nsnull;

  if ( NULL == aResult )
  {
    rv = NS_ERROR_NULL_POINTER;
    return rv;
  }
  *aResult = NULL;
  if (NULL != aOuter)
  {
    rv = NS_ERROR_NO_AGGREGATION;
    return rv;
  }
  
  inst = (nsISupports *)(nsBaseWidget *)(nsWidget *)new nsScrollbar(PR_FALSE);
  if (inst == NULL)
  {
    return NS_ERROR_OUT_OF_MEMORY;
  }
  NS_ADDREF(inst);
  rv = inst->QueryInterface(aIID, aResult);
  NS_RELEASE(inst);

  return rv;
}

static nsresult nsVertScrollbarConstructor (nsISupports *aOuter, REFNSIID aIID, void **aResult)
{
  nsresult rv;
  nsISupports *inst = nsnull;

  if ( NULL == aResult )
  {
    rv = NS_ERROR_NULL_POINTER;
    return rv;
  }
  *aResult = NULL;
  if (NULL != aOuter)
  {
    rv = NS_ERROR_NO_AGGREGATION;
    return rv;
  }
  
  inst = (nsISupports *)(nsBaseWidget *)(nsWidget *)new nsScrollbar(PR_TRUE);
  if (inst == NULL)
  {
    return NS_ERROR_OUT_OF_MEMORY;
  }
  NS_ADDREF(inst);
  rv = inst->QueryInterface(aIID, aResult);
  NS_RELEASE(inst);

  return rv;
}

static nsModuleComponentInfo components[] =
{
  { "NanoX nsWindow",
    NS_WINDOW_CID,
    "@mozilla.org/widgets/window/nanox;1",
    nsWindowConstructor },
  { "NanoX Child nsWindow",
    NS_CHILD_CID,
    "@mozilla.org/widgets/child_window/nanox;1",
    ChildWindowConstructor },
  //{ "NanoX Button",
    //NS_BUTTON_CID,
    //"@mozilla.org/widgets/button/nanox;1",
    //nsButtonConstructor },
  //{ "NanoX Check Button",
    //NS_CHECKBUTTON_CID,
    //"@mozilla.org/widgets/checkbutton/nanox;1",
    //nsCheckButtonConstructor },
  //{ "NanoX File Widget",
    //NS_FILEWIDGET_CID,
    //"@mozilla.org/widgets/filewidget/nanox;1",
    //nsFileWidgetConstructor },
  { "NanoX Horiz Scrollbar",
    NS_HORZSCROLLBAR_CID,
    "@mozilla.org/widgets/horizscroll/nanox;1",
    nsHorizScrollbarConstructor },
  { "NanoX Vert Scrollbar",
    NS_VERTSCROLLBAR_CID,
    "@mozilla.org/widgets/vertscroll/nanox;1",
    nsVertScrollbarConstructor },
  //{ "NanoX Text Widget",
    //NS_TEXTFIELD_CID,
    //"@mozilla.org/widgets/textwidget/nanox;1",
    //nsTextWidgetConstructor },
  { "NanoX AppShell",
    NS_APPSHELL_CID,
    "@mozilla.org/widget/appshell/nanox;1",
    nsAppShellConstructor },
  { "NanoX Toolkit",
    NS_TOOLKIT_CID,
    "@mozilla.org/widget/toolkit/nanox;1",
    nsToolkitConstructor },
  { "NanoX Look And Feel",
    NS_LOOKANDFEEL_CID,
    "@mozilla.org/widget/lookandfeel/nanox;1",
    nsLookAndFeelConstructor },
  //{ "NanoX Label",
    //NS_LABEL_CID,
    //"@mozilla.org/widget/label/nanox;1",
    //nsLabelConstructor },
  { "NanoX Sound",
    NS_SOUND_CID,
      "@mozilla.org/widget/sound/nanox;1",
    //"@mozilla.org/sound;1",
    nsSoundConstructor },
  { "Transferrable",
    NS_TRANSFERABLE_CID,
    //    "@mozilla.org/widget/transferrable/nanox;1",
    "@mozilla.org/widget/transferable;1",
    nsTransferableConstructor },
  { "NanoX Clipboard",
    NS_CLIPBOARD_CID,
    //    "@mozilla.org/widget/clipboard/nanox;1",
    "@mozilla.org/widget/clipboard;1",
    nsClipboardConstructor },
  { "HTML Format Converter",
    NS_HTMLFORMATCONVERTER_CID,
    "@mozilla.org/widget/htmlformatconverter/nanox;1",
    nsHTMLFormatConverterConstructor },
  //{ "NanoX Font Retriever Service",
    //NS_FONTRETRIEVERSERVICE_CID,
    //"@mozilla.org/widget/fontretrieverservice/nanox;1",
    //nsFontRetrieverServiceConstructor },
  { "NanoX Drag Service",
    NS_DRAGSERVICE_CID,
    //    "@mozilla.org/widget/dragservice/nanox;1",
    "@mozilla.org/widget/dragservice;1",
    nsDragServiceConstructor },
#ifdef IBMBIDI
  { "Xlib Bidi Keyboard",
    NS_BIDIKEYBOARD_CID,
    "@mozilla.org/widget/bidikeyboard;1",
    nsBidiKeyboardConstructor },
#endif // IBMBIDI
  { "File Spec with UI",
    NS_FILESPECWITHUI_CID,
    //    "@mozilla.org/widget/filespecwithui/nanox;1",
    "@mozilla.org/filespecwithui;1",
    nsFileSpecWithUIImplConstructor }
};

NS_IMPL_NSGETMODULE("nsWidgetNANOXModule", components)
