/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s):
 *   Peter Hartshorn <peter@igelaus.com.au>
 *   B.J. Rossiter <bj@igelaus.com.au>
 */

#ifndef nsINanoXWindowService_h__
#define nsINanoXWindowService_h__

#include "nsISupports.h"

#include <sys/time.h>

// Interface id for the NanoXWindow service
// { bd39ccb0-3f08-11d3-b419-00805f6d4c2a }
#define NS_NANOX_WINDOW_SERVICE_IID        \
              { 0xbd39ccb0,               \
              0x3f08,                     \
              0x11d3,                     \
              { 0xb4, 0x19, 0x00, 0x80, 0x5f, 0x6d, 0x4c, 0x2a } }

// Class ID for our implementation
// { 285fb550-3af4-11d3-aeb9-0000f8e25c06 }
#define NS_NANOX_WINDOW_SERVICE_CID \
  { 0x285fb550,                    \
    0x3af4,                        \
    0x11d3,                        \
    { 0xae, 0xb9, 0x00, 0x00, 0xf8, 0xe2, 0x5c, 0x06 } }

#define NS_NANOX_WINDOW_SERVICE_PROGID "component://netscape/widget/nanox/window_service"

typedef void (*nsNanoXWindowCallback)(PRUint32 aWindowID);

typedef void * nsNanoXNativeEvent;

typedef void (*nsNanoXEventDispatcher)(nsNanoXNativeEvent aNativeEvent);

typedef int (*nsNanoXTimeToNextTimeoutFunc)(struct timeval *aTimer);

typedef void (*nsNanoXProcessTimeoutsProc)(void);

class nsINanoXWindowService : public nsISupports
{
 public:
  NS_DEFINE_STATIC_IID_ACCESSOR(NS_NANOX_WINDOW_SERVICE_IID);

  NS_IMETHOD SetWindowCreateCallback(nsNanoXWindowCallback aCallback) = 0;
  NS_IMETHOD SetWindowDestroyCallback(nsNanoXWindowCallback aCallback) = 0;
  NS_IMETHOD GetWindowCreateCallback(nsNanoXWindowCallback * aCallbackOut) = 0;
  NS_IMETHOD GetWindowDestroyCallback(nsNanoXWindowCallback * aCallbackOut) = 0;

  NS_IMETHOD SetEventDispatcher(nsNanoXEventDispatcher aDispatcher) = 0;
  NS_IMETHOD GetEventDispatcher(nsNanoXEventDispatcher * aDispatcherOut) = 0;

  NS_IMETHOD SetTimeToNextTimeoutFunc(nsNanoXTimeToNextTimeoutFunc aFunc) = 0;
  NS_IMETHOD GetTimeToNextTimeoutFunc(nsNanoXTimeToNextTimeoutFunc * aFuncOut) = 0;

  NS_IMETHOD SetProcessTimeoutsProc(nsNanoXProcessTimeoutsProc aProc) = 0;
  NS_IMETHOD GetProcessTimeoutsProc(nsNanoXProcessTimeoutsProc * aProcOut) = 0;
  
};

#endif /* nsINanoXWindowService_h__ */
